Use *stubbs:add-option* to add an option to a command.

Basic usage
-----------

After creating a command with 
[stubbs:add-command](../add-command/index.html),
use stubbs:add-option to define a command option. 
The add-option command takes several arguments that specify
the various option properties.

Below the `--jumps` option is added to the command waitfor:ping.

    rerun stubbs:add-option --option jumps --description "the host to reach" --module waitfor --command ping --required true --export false --default '""'

You will see output similar to:

    Created option: /Users/alexh/.rerun/modules/waitfor/options/jumps/metadata
    Updated command metadata:  /Users/alexh/.rerun/modules/waitfor/commands/ping/metadata

Besides the `options/jumps/metadata` file, the `add-option` 
command also generates an option parsing function contained
in the modules function library.

The ping command's `script` calls this option parsing function 
to read the command line arguments and set shell variables
accessible to the command script.

Users will now be able to specify a "--host" option
to the `waitfor:ping` command:

    $ rerun waitfor
    waitfor:
     ping: wait for ping response from host
        --host <>: "the host to reach"

* Use the [stubbs:edit](../edit/index.html) command to 
open the generated `script` file in the text 
editor referenced via the `$EDITOR` environment variable. Eg,

    rerun stubbs: edit --module waitfor --command ping 

* See the [stubbs:rm-option](../rm-option) command to remove an option.
If a command option is not supplied by the user, 
the options parser script (generated by add-option) can set a default value.

Add an option that has a default
--------------------------------

Call the "add-option" command again using 
its `--default <>` parameter to set a default value.

Here the "--interval" option is set to use a default value, "30":

    rerun stubbs:add-option \
      --option interval -description "seconds between checks" \
      --module waitfor --command ping \
      --required false --export false \
      --default 30

The add-option will update the metadata for the 
"interval" option.

List the waitfor commands to see "ping" and the new 
"--interval" option:

    $ rerun waitfor
    ping: "tell waitfor to ping"
        --host <>: "the host to reach"
        [ --interval <30>]: "seconds between checks"

Add an option to multiple commands
----------------------------------

The "--command" option can take a comma separated list
of command names. This is useful if commands in your module
need to share the same option.

    rerun stubbs:add-option \
      --option interval -description "seconds between checks" \
      --module waitfor --command ping,pgrep,nc \
      --required false --export false \
      --default 30

Commands that don't exist are skipped.


Internal details
----------------

You might be interested in the options.sh script that's 
created behind the scenes. Below, the "ping" command's 
options.sh script is shown.
The meat of the script is the while loop and case statement. 
In the body of the case statement, you can see a case 
for the "--host" option and the `HOST` variable that will 
be set to the value of the "--host" argument.

    # generated by add-option
    # Tue Sep 13 20:11:52 PDT 2011

    rerun_options_parse() {
     
        while [ "$#" -gt 0 ]; do
            OPT="$1"
            case "$OPT" in
                --host) rerun_option_check $# ; HOST=$2 ; shift ;;
                --interval) rerun_option_check $# ; INTERVAL=$2 ; shift ;;
                # help option
                -?)
                    rerun_option_usage
                    exit 2
                    ;;
                # end of options, just arguments left
                *)
                  break
            esac
            shift
        done
     
        # Set defaultable options.
        [ -z "$INTERVAL" ] && INTERVAL="30"
        # Check required options are set
        [ -z "$HOST" ] && { echo >&2 "missing required option: --host" ; return 2 ; }
        #
        return 0
    }

Below the while loop, you can see a test for the `INTERVAL`
variable (check for empty string). A statement like this 
is added for options that declare `DEFAULT` metadata property.

Separating options processing into the options.sh script, 
away from the command implementation logic in script, 
keeps the options parsing clutter out of your way.
